// $Id: timesize.cpp,v 1.5 2006/10/17 02:10:58 erynes Exp $

/* 
   Copyright 2002  Peter Beerli, Mary Kuhner, Jon Yamato and Joseph Felsenstein

   This software is distributed free of charge for non-commercial use
   and is copyrighted.  Of course, we do not guarantee that the software
   works, and are not responsible for any damage you may cause or have.
*/

#include "force.h"
#include "mathx.h"    // for SafeProductWithExp()
#include "timesize.h"


#ifdef DMALLOC_FUNC_CHECK
#include "/usr/local/include/dmalloc.h"
#endif

using namespace std;


DoubleVec1d GrowTimeSize::PartitionSizeAt(force_type partforce, 
				 const DoubleVec1d& thetas, const DoubleVec1d& growths, double etime) const
{
  return dynamic_cast<PartitionForce*>(*registry.GetForceSummary().
				      GetForceByTag(partforce))->SumXPartsToParts(thetas,growths,etime);
} // GrowTimeSize::PartitionSizeAt

//___________________________________________________________

DoubleVec1d GrowTimeSize::SizeAt(const DoubleVec1d& thetas, const
   DoubleVec1d& growths, double etime) const
{
  assert(thetas.size() == growths.size());
  DoubleVec1d newthetas(thetas.size());
  unsigned long param;
  for(param = 0; param < thetas.size(); ++param) {
     newthetas[param] = SafeProductWithExp(thetas[param],-growths[param]*etime);
  }

  return newthetas;
} // GrowTimeSize::SizeAt

//___________________________________________________________
//___________________________________________________________

DoubleVec1d StaticTimeSize::PartitionSizeAt(force_type partforce,
				   const DoubleVec1d& thetas, const DoubleVec1d& growths, double etime) const
{
  return dynamic_cast<PartitionForce*>(*registry.GetForceSummary().
		      GetForceByTag(partforce))->SumXPartsToParts(thetas);
} // StaticTimeSize::PartitionSizeAt

//___________________________________________________________

DoubleVec1d StaticTimeSize::SizeAt(const DoubleVec1d& thetas, const
   DoubleVec1d& growths, double etime) const
{
  return thetas;
} /* StaticTimeSize::SizeAt */

//___________________________________________________________

DoubleVec1d LogisticSelectionTimeSize::SizeAt(const DoubleVec1d& thetas, const
   DoubleVec1d& growths, double etime) const
{
  if (2 != thetas.size())
    {
      string msg = "LogisticSelectionTimeSize::SizeAt(), needed to receive ";
      msg += "two thetas, one for each allele, but instead received ";
      msg += ToString(thetas.size());
      msg + " thetas.";
      throw implementation_error(msg);
    }
  if (!growths.empty())
    {
      string msg = "LogisiticSelectionTimeSize::SizeAt(), received a non-empty ";
      msg += "vector of population growth rates.  Logistic selection is currently ";
      msg += "limited to fixed-size populations.  If other parts of LAMARC imply ";
      msg += "otherwise, then this method needs to be updated to accomodate growth.";
      throw implementation_error(msg);
    }

  DoubleVec1d newthetas(2);
  double onePlusS_allToThe_t = pow(1. + m_selectionCoefficient, etime);
  newthetas[0] = (thetas[0]+thetas[1])*thetas[0]/(thetas[0] + thetas[1]*onePlusS_allToThe_t);
  newthetas[1] = newthetas[0]*thetas[1]*onePlusS_allToThe_t/thetas[0];

  return newthetas;
} // LogisticSelectionTimeSize::SizeAt

DoubleVec1d LogisticSelectionTimeSize::PartitionSizeAt(force_type partforce, 
	 const DoubleVec1d& thetas, const DoubleVec1d& growths, double etime) const
{
  DoubleVec1d emptyvec;
  throw implementation_error("LogisticSelectionTimeSize::PartitionSizeAt() was called; shouldn\'t be?");
  return emptyvec;
} // LogisticSelectionTimeSize::PartitionSizeAt

