#include "guiconverter.h"
#include "gc_cmdline.h"
#include "gc_event_ids.h"
#include "gc_event_publisher.h"
#include "gc_frame.h"
#include "gc_layout.h"
#include "gc_strings.h"

#include "giraffe32.xpm"

#include "tinyxml.h"

#include "wx/cmdline.h"
#include "wx/icon.h"
#include "wx/log.h"

GuiConverterApp::GuiConverterApp()
{
}

GuiConverterApp::~GuiConverterApp()
{
}


IMPLEMENT_APP(GuiConverterApp)

void
GuiConverterApp::OnInitCmdLine(wxCmdLineParser& parser)
{
    wxApp::OnInitCmdLine(parser);
    GCCmdLineManager::AddOptions(parser);
}

bool
GuiConverterApp::OnCmdLineParsed(wxCmdLineParser& parser)
{
    bool parentReturned = wxApp::OnCmdLineParsed(parser);
    GCCmdLineManager::ExtractValues(parser);
    return parentReturned;
}

bool
GuiConverterApp::OnInit()
{


    // use the parent's OnInit because it includes command line parsing
    if(wxApp::OnInit())
    {
        wxLog::SetActiveTarget(new wxLogStderr());

        GCCmdLineManager::ProcessCommandLineAndCommandFile(m_logic);
        if(m_batchOnly)
        {
            GCCmdLineManager::DoExport(m_logic);
            wxExit();
            return false;
        }
        else
        {

            int appWidth  = gclayout::appWidth;
            int appHeight = gclayout::appHeight;
            wxSize maxSize = wxGetDisplaySize();
            int maxWidth = maxSize.GetWidth()  * gclayout::appWidthPercent  / 100;
            int maxHeight= maxSize.GetHeight() * gclayout::appHeightPercent / 100;
            if(appWidth > maxWidth)
            {
                appWidth = maxWidth;
            }
            if(appHeight > maxHeight)
            {
                appHeight = maxHeight;
            }

            wxSize bestSize(appWidth,appHeight);

            GCFrame * m_mainFrame = new GCFrame(gcstr::converterTitle,m_logic);
            m_logic.SetDisplayParent(m_mainFrame);
            m_mainFrame->SetSize(bestSize);
            m_mainFrame->CentreOnScreen();
            m_mainFrame->Show(true);
            SetTopWindow(m_mainFrame);

            m_mainFrame->SetIcon( wxICON( giraffe32) );



            PublishDataEvent(m_mainFrame->GetEventHandler(),D2S_UserInteractionPhaseEnd);

            return true;
        }
    }
    return false;

}

int
GuiConverterApp::OnRun()
{
    try
    {
        int result = wxApp::OnRun();
        return result;
    }
    catch(const std::exception& e)
    {
        m_logic.GCFatal(wxString::Format(gcerr::uncaughtException,e.what()));
    }
    return 1;       // EWFIX.P3 -- what should this be?
}


int
GuiConverterApp::OnExit()
{
    if(m_doDebugDump)
    {
        m_logic.DebugDump();
    }
    if(! m_batchOutName.IsEmpty())
    {
        TiXmlDocument * doc = m_logic.ExportBatch();
        m_logic.WriteBatchFile(doc,m_batchOutName);
        delete doc;
    }

    m_logic.NukeContents();

    return wxApp::OnExit();
}

